/**
 * EasyBeans
 * Copyright (C) 2006-2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Greeter.java 2546 2008-02-29 13:49:31Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.examples.cxf;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.Local;
import javax.ejb.SessionContext;
import javax.ejb.Stateless;
import javax.interceptor.AroundInvoke;
import javax.interceptor.Interceptors;
import javax.interceptor.InvocationContext;
import javax.jws.HandlerChain;
import javax.jws.WebService;
import javax.xml.ws.WebServiceContext;

import org.ow2.easybeans.cxf.PortComponent;
import org.ow2.easybeans.examples.cxf.gen.IGreeter;

/**
 * Local Stateless Session Bean exposed as WebService.
 * @author Guillaume Sauthier
 */
@Stateless
@Local(ISayHi.class)
// Defines that instance as a WebService
@WebService(name="Greeter",
            serviceName="SOAPService",
            targetNamespace="http://easybeans.ow2.org/examples/cxf/greeter",
            wsdlLocation="META-INF/wsdl/greeter.wsdl")
// EasyBeans only !
@PortComponent(urlPattern="/greeter")
// Defines an external EJB3 interceptor/callback class
@Interceptors(GreeterInterceptor.class)
// Defines a HandlerChain (aka WS interceptors) for that instance.
@HandlerChain(file="config/greeter-chain.xml")
public class Greeter implements IGreeter, ISayHi {

    /**
     * SessionContext Resource. Will be injected by EasyBeans.
     */
    @Resource
    private SessionContext sessionContext;

    /**
     * WebServiceContext Resource. Will be injected by CXF.
     */
    @Resource
    private WebServiceContext wsContext;


    /**
     * Method exposed as WebService.
     * @param me the name of the person to greet.
     * @return a string computed with the person's name
     */
    public String greetMe(final String me) {
        System.out.println("Message received: " + me + "\n");

        // check if the SessionContext was injected
        if (this.sessionContext != null) {
            System.out.println("SessionContext was injected by Easybeans.");
        }
        // Check if the WebServiceContext was injected
        if (this.wsContext != null) {
            System.out.println("WebServiceContext was injected by CXF.");
        }

        return "Hello " + me;
    }

    /**
     * Method exposed as WebService.
     * @return the Bonjour text
     */
    public String sayHi() {
        System.out.println("Executing operation sayHi\n");
        return "Bonjour";
    }

    // -----------------------------------------------------
    // Interceptors and LifeCycle methods
    // ----------------------------------------------------

    /**
     * Bean level Interceptor.
     * @param ic the invocation context object
     * @throws Exception if the interceptor fails
     * @return the result of the invocation
     */
    @AroundInvoke
    protected Object trace(final InvocationContext ic) throws Exception {
        String mName = ic.getMethod().getName();
        System.out.println("Entering method : " + mName);
        Object o = ic.proceed();
        System.out.println("Leaving method : " + mName);
        return o;
    }

    /**
     * LifeCycle callback, called after Bean creation.
     */
    @PostConstruct
    public void post() {
        System.out.println("PostConstruct method invoked ...");
    }
}
